@@ -25,7 +25,7 @@
 
 (defun guess-alignment (bits)
   (cond ((null bits) nil)
-        #-(and x86 (not win32)) ((> bits 32) 64)
+        #-(or (and x86 (not win32)) (and ppc darwin)) ((> bits 32) 64)
         ((> bits 16) 32)
         ((> bits 8) 16)
         ((> bits 1) 8)
@@ -178,10 +178,10 @@
             `((declaim (inline ,constructor))
               (defun ,constructor (&key ,@constructor-args)
                 (let ((hash ,(logior (ash (alien-type-class-name->id name) type-hash-nbits)
-                                     (ldb (byte type-hash-nbits 0) (sb-xc:sxhash name)))))
+                                     (ldb (byte type-hash-nbits 0) (symbol-name-hash name)))))
                   (,allocator hash ,@allocator-args)))))))))
 
-(defmacro define-alien-type-method ((class method) lambda-list &rest body)
+(defmacro define-alien-type-method ((class method) lambda-list &body body)
   (let ((defun-name (symbolicate class "-" method "-METHOD")))
     `(progn
        (defun ,defun-name ,lambda-list
@@ -520,18 +520,24 @@
   ;; of return values and override the naturalize method to perform
   ;; the sign extension (in compiler/{arch}/c-call.lisp).
   (ecase context
-    ((:normal #-(or x86 x86-64) :result)
+    ((:normal #-(or x86 x86-64 loongarch64 riscv) :result)
      (list (if (alien-integer-type-signed type) 'signed-byte 'unsigned-byte)
            (alien-integer-type-bits type)))
     #+(or x86 x86-64)
     (:result
      (list (if (alien-integer-type-signed type) 'signed-byte 'unsigned-byte)
            (max (alien-integer-type-bits type)
-                sb-vm:n-machine-word-bits)))))
+                sb-vm:n-machine-word-bits)))
+    #+(or loongarch64 riscv)
+    (:result
+     (list (if (alien-integer-type-signed type) 'signed-byte 'unsigned-byte)
+           (if (>= (alien-integer-type-bits type) 32)
+               sb-vm:n-machine-word-bits
+               (alien-integer-type-bits type))))))
 
 ;;; As per the comment in the :ALIEN-REP method above, this is defined
-;;; elsewhere for x86oids.
-#-(or x86 x86-64)
+;;; elsewhere
+#-(or x86 x86-64 riscv loongarch64)
 (define-alien-type-method (integer :naturalize-gen) (type alien)
   (declare (ignore type))
   alien)
@@ -621,7 +627,7 @@
            ;; * SIGNED is true if and only if the minimum value is negative
            ;; * OFFSET is based on the minimum value if and only if the
            ;;   inverse map is stored as is a vector
-       `(let ((h (sb-xc:sxhash name)))
+       `(let ((h (symbol-name-hash name)))
           (dolist (elt (alien-enum-type-from result))
             ;; Mix by hand since our SXHASH has a cutoff on length, not to
             ;; mention that this potentially runs on either the host or the target,
@@ -1031,7 +1037,7 @@
            (hash-sym (symbol)
              ;; Since ALIEN-TYPE literals are reconstructed via load-forms that involve
              ;; the constructor, host object hashes don't have to be target-compatible.
-             #+sb-xc-host (sb-xc:sxhash symbol)
+             #+sb-xc-host (symbol-name-hash symbol)
              ;; This mixes in package-id when hashing field names.
              ;; The reason is that if you have 100 different packages each defining
              ;; (STRUCT FOO (WORD0 INT) (WORD1 INT)), then without a better hash
@@ -1040,8 +1046,9 @@
              ;; a symbol. It's not important because at worst, you miss in the hashset.
              ;; We can't perfectly hash-cons record types anyway.
              ;; (Detecting isomorphism between possibly cyclic objects is hard)
+             ;; Question: should this just use SYMBOL-HASH instead?
              #-sb-xc-host (mix (sb-kernel:symbol-package-id symbol)
-                               (sb-xc:sxhash symbol))))
+                               (symbol-name-hash symbol))))
     (mix (hash-sym (alien-record-type-name r))
          (hash-fields (alien-record-type-fields r)))))
 (defun alien-record-type-equiv (a b)
@@ -1148,7 +1155,13 @@
               (setf (auxiliary-alien-type kind name env)
                     (!make-alien-record-type :name name :kind kind))))
          (t
-          (!make-alien-record-type :kind kind)))))
+          (ecase kind
+            (:union
+             (load-time-value
+              (!make-alien-record-type :kind :union)))
+            (:struct
+             (load-time-value
+              (!make-alien-record-type :kind :struct))))))))
   (define-alien-type-translator union (name &rest fields &environment env)
     (parse-alien-record-type :union name fields env))
   (define-alien-type-translator struct (name &rest fields &environment env)
@@ -1266,6 +1279,7 @@
   ;; as indicative of "..." in the C prototype. We can record that too.
   (varargs nil :type (or boolean fixnum (eql :unspecified)) :read-only t)
   (stub nil :type (or null function))
+  (into-stub nil :type (or null function)) ; for alien-funcall-into
   (convention nil :type calling-convention :read-only t))
 ;;; The safe default is to assume that everything is varargs.
 ;;; On x86-64 we have to emit a spurious instruction because of it.
@@ -1493,13 +1507,15 @@
                ((alien-value mem-block) 0)
                (pointer '(if to (alien-type-hash to) #xBAD))
                (c-string ; Don't need ALIEN-POINTER-TYPE-TO as it's invariant
-                (mix* (sb-xc:sxhash element-type) (sb-xc:sxhash external-format)
-                      (sb-xc:sxhash not-null)))
+                (mix* (sb-kernel:symbol-hash element-type)
+                      (sb-kernel:symbol-hash external-format)
+                      (sb-kernel:symbol-hash not-null)))
                ((integer boolean) '(if signed 1 0))
                (array `(let ((h (alien-type-hash element-type)))
                          (dolist (dim dimensions h) (setf h (mix dim h)))))
                (fun (mix* (alien-type-hash result-type) (hash-alien-type-list arg-types)
-                          (sb-xc:sxhash varargs) (sb-xc:sxhash convention)))
+                          (sb-xc:sxhash varargs)
+                          (sb-kernel:symbol-hash convention)))
                (values '(hash-alien-type-list values))))))
        (collect ((forms) (globals))
          (dolist (list *hashset-defining-forms*
@@ -1571,6 +1587,8 @@
 (defun double-float-type () (parse-alien-type 'double-float nil))
 (defun sap-type () (parse-alien-type 'system-area-pointer nil))
 
+(declaim (freeze-type alien-type))
+
 ;;; Genesis is limited in its ability to patch load-time-values into objects
 ;;; other than code components. For this reason we want to emit a single sexpr
 ;;; to handle all levels of structure nesting.
