@@ -159,14 +159,19 @@
   (define ftruncate cl:truncate))
 
 (defun xfloat-expt (base power)
-  (if (not (integerp power))
-      (error "Unimplemented: EXPT with non-integer power")
-      (with-memoized-math-op (expt (list base power))
-        (if (zerop power)
-            (coerce 1 (type-of base))
-            (flonum-from-rational
-             (cl:expt (rational base) power)
-             (pick-result-format base power))))))
+  (when (sb-kernel::integer-float-p power)
+    (setf power (truncate power)))
+  (cond ((not (integerp power))
+         (error "Unimplemented: EXPT with non-integer power ~a" power))
+        (t
+         (with-memoized-math-op (expt (list base power))
+           (if (zerop power)
+               (if (integerp base)
+                   1
+                   (coerce 1 (type-of base)))
+               (flonum-from-rational
+                (cl:expt (rational base) power)
+                (pick-result-format base power)))))))
 
 ;;; Four possible return values.  NIL if the numbers (rationals or
 ;;; flonums) are incomparable (either is a NaN).  Otherwise: -1, 0, 1
@@ -314,6 +319,10 @@
                 (error "can't represent NaN for (/ 0 0)"))
                ((zerop y)
                 (error "can't represent Inf for (/ x 0)"))
+               ((and (floatp x)
+                     (floatp y)
+                     (float-infinity-p y))
+                (coerce (if (cl:= (sgn x) (sgn y)) 0 -0.0) format))
                ((zerop x)
                 (coerce (if (cl:= (sgn x) (sgn y)) 0 -0.0) format))
                (t (flonum-from-rational (cl:/ (rational x) (rational y)) format))))))
@@ -335,6 +344,7 @@
                (setf current (cl:/ (cl:+ current (cl:/ rational current)) 2))))))
     (%%sqrt rational (cl:/ (isqrt (numerator rational)) (isqrt (denominator rational))))))
 
+#-c-headers-only
 (defun sb-xc:sqrt (arg)
   (cond ((eql arg 0)
          0.0)
@@ -355,6 +365,7 @@
   `(progn ,@body))
 
 (defun realpart (x) (if (realp x) x (complexnum-real x)))
+#-c-headers-only
 (defun imagpart (x)
   (cond ((rationalp x) 0)
         ((single-float-p x) 0f0)
@@ -366,6 +377,7 @@
 
 ;;; PI is needed in order to build the cross-compiler mainly so that vm-fndb
 ;;; can define bounds on irrational functions.
+#-c-headers-only
 (defconstant pi 3.14159265358979323846264338327950288419716939937511L0)
 
 (macrolet ((def (name lambda-list)
@@ -383,6 +395,7 @@
   (def tan (number))
   (def tanh (number)))
 
+#-c-headers-only
 (defun asin (number)
   (case number
     (-1d0 -1.5707963267948966d0)
@@ -392,6 +405,7 @@
     (t
      (error "Unimplemented."))))
 
+#-c-headers-only
 (defun acos (number)
   (case number
     (-1d0 pi)
@@ -401,6 +415,7 @@
     (t
      (error "Unimplemented."))))
 
+#-c-headers-only
 (defun acosh (number)
   (with-memoized-math-op (acosh number)
     (case number
@@ -409,6 +424,7 @@
       (t
        (error "Unimplemented.")))))
 
+#-c-headers-only
 (defun atanh (number)
   (with-memoized-math-op (atanh number)
     (case number
@@ -419,6 +435,7 @@
       (t
        (error "Unimplemented.")))))
 
+#-c-headers-only
 (defun atan (number1 &optional (number2 nil number2p))
   (if number2p
       (with-memoized-math-op (atan (list number1 number2))
@@ -428,6 +445,7 @@
             number1
             (error "Unimplemented.")))))
 
+#-c-headers-only
 (defun cosh (number)
   (with-memoized-math-op (cosh number)
     (case number
@@ -542,6 +560,7 @@
                  table)))))
 
 ;;; Perform some simple checks
+#-c-headers-only (progn
 (assert (not (eq -0.0f0 -0.0d0)))
 (assert (not (eq single-float-negative-infinity 0f0)))
 (dolist (format '(single-float double-float))
@@ -595,7 +614,7 @@
   (flet ((assert-not-number (x)
            (handler-case (rational x)
              (:no-error (x) (error "Expected an error, got ~S" x))
-             (simple-error (x) (declare (ignore x))))))
+             (floating-point-invalid-operation ()))))
     (let ((nan (make-single-float #b01111111101000000000000000000000)))
       ;;                             [ exp  ]
       (assert-not-number nan)
@@ -606,3 +625,4 @@
     (assert-not-number single-float-positive-infinity)
     (assert-not-number double-float-negative-infinity)
     (assert-not-number double-float-positive-infinity))))
+)
