@@ -90,11 +90,32 @@
 
 ;;;; list collection macrology
 
+(defstruct (loop-collector
+            (:copier nil)
+            (:predicate nil))
+  (name nil :read-only t)
+  (class nil :read-only t)
+  (history nil)
+  (tempvars nil)
+  (specified-type nil :read-only t)
+  dtype
+  (data nil)) ;collector-specific data
+(declaim (sb-ext:freeze-type loop-collector))
+
 (sb-xc:defmacro with-loop-list-collection-head
-    ((head-var tail-var &optional user-head-var) &body body)
-  (let ((l (and user-head-var (list (list user-head-var nil)))))
-    `(let* ((,head-var (list nil)) (,tail-var ,head-var) ,@l)
-       (declare (truly-dynamic-extent ,head-var))
+    ((collector head-var tail-var &optional user-head-var) &body body &environment env)
+  (let ((l (and user-head-var (list (list user-head-var nil))))
+        (debug (sb-c:policy env (= debug 3))))
+    `(let* ((,head-var ,(if (and (not debug)
+                                 (loop for how in (loop-collector-history collector)
+                                       always (eq how 'list)))
+                            `(unaligned-dx-cons nil)
+                            `(list nil)))
+            (,tail-var ,head-var) ,@l)
+       (declare (dynamic-extent ,head-var)
+                ,@(unless debug
+                    `((sb-c::no-debug ,head-var ,tail-var))) ;; half-conses are bad for the debugger
+                ,@(and user-head-var `((list ,user-head-var))))
        ,@body)))
 
 (sb-xc:defmacro loop-collect-rplacd
@@ -120,8 +141,10 @@
                  (setq ncdrs (- (length (cdr form)) 2))))))
       (let ((answer
               (cond ((null ncdrs)
-                     `(when (setf (cdr ,tail-var) ,tail-form)
-                        (setq ,tail-var (last (cdr ,tail-var)))))
+                     (if (typep tail-form '(cons (eql copy-list)))
+                         `(setf ,tail-var (sb-impl::copy-list-to ,(second tail-form)  ,tail-var))
+                         `(when (setf (cdr ,tail-var) ,tail-form)
+                            (setq ,tail-var (last (cdr ,tail-var))))))
                     ((< ncdrs 0) (return-from loop-collect-rplacd nil))
                     ((= ncdrs 0)
                      ;; @@@@ Here we have a choice of two idioms:
@@ -140,13 +163,13 @@
         (when user-head-var
           (setq answer
                 `(progn ,answer
-                        (setq ,user-head-var (cdr ,head-var)))))
+                        (setq ,user-head-var (sb-ext:truly-the list (cdr ,head-var))))))
         answer))))
 
 (sb-xc:defmacro loop-collect-answer (head-var
-                                                   &optional user-head-var)
-  (or user-head-var
-      `(cdr ,head-var)))
+                                     &optional user-head-var)
+  `(sb-ext:truly-the list ,(or user-head-var
+                               `(cdr ,head-var))))
 
 ;;;; maximization technology
 
@@ -189,9 +212,8 @@ constructed.
     (make-loop-minimax-internal
       :answer-variable answer-variable
       :type type
-      :temp-variable (gensym "LOOP-MAXMIN-TEMP-")
-      :flag-variable (and (not infinity-data)
-                          (gensym "LOOP-MAXMIN-FLAG-"))
+      :temp-variable (gensym "MINMAXTMP")
+      :flag-variable (and (not infinity-data) (gensym "MINMAXFLAG"))
       :operations nil
       :infinity-data infinity-data)))
 
@@ -199,8 +221,7 @@ constructed.
   (pushnew (the symbol operation) (loop-minimax-operations minimax))
   (when (and (cdr (loop-minimax-operations minimax))
              (not (loop-minimax-flag-variable minimax)))
-    (setf (loop-minimax-flag-variable minimax)
-          (gensym "LOOP-MAXMIN-FLAG-")))
+    (setf (loop-minimax-flag-variable minimax) (gensym "MINMAXFLAG")))
   operation)
 
 (sb-xc:defmacro with-minimax-value (lm &body body)
@@ -289,10 +310,10 @@ code to be loaded.
              (:constructor !make-loop-universe)
              (:copier nil)
              (:predicate nil))
-  keywords             ; hash table, value = (fn-name . extra-data)
-  iteration-keywords   ; hash table, value = (fn-name . extra-data)
-  for-keywords         ; hash table, value = (fn-name . extra-data)
-  path-keywords)       ; hash table, value = (fn-name . extra-data)
+  (keywords nil :read-only t)             ; hash table, value = (fn-name . extra-data)
+  (iteration-keywords nil :read-only t)   ; hash table, value = (fn-name . extra-data)
+  (for-keywords nil :read-only t)         ; hash table, value = (fn-name . extra-data)
+  (path-keywords nil :read-only t))       ; hash table, value = (fn-name . extra-data)
 (declaim (sb-ext:freeze-type loop-universe))
 (defmethod print-object ((u loop-universe) stream)
   (print-unreadable-object (u stream :type t :identity t)))
@@ -324,9 +345,6 @@ code to be loaded.
 (defun loop-make-desetq (var-val-pairs)
   (if var-val-pairs (cons 'loop-desetq var-val-pairs)))
 
-(sb-ext:defglobal *loop-desetq-temporary*
-        (make-symbol "LOOP-DESETQ-TEMP"))
-
 (sb-xc:defmacro loop-desetq (&environment env &rest var-val-pairs)
   (labels ((find-non-null (var)
              ;; See whether there's any non-null thing here. Recurse
@@ -380,7 +398,8 @@ code to be loaded.
                    (when (or car-non-null cdr-non-null)
                      (if cdr-non-null
                          (let* ((temp-p temp)
-                                (temp (or temp *loop-desetq-temporary*))
+                                (temp (or temp (load-time-value
+                                                (make-symbol "DSTEMP") t)))
                                 (body `(,@(loop-desetq-internal car
                                                                 `(car ,temp))
                                           (setq ,temp (cdr ,temp))
@@ -561,8 +580,7 @@ code to be loaded.
   (do ((l (source-context loop) (cdr l)) (new nil (cons (car l) new)))
       ((eq l (cdr (source-code loop))) (nreverse new))))
 
-(defun loop-error (format-string &rest format-args)
-  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper loop-error (format-string &rest format-args)
   (%program-error "~?~%current LOOP context:~{ ~S~}."
                   format-string format-args (loop-context)))
 
@@ -595,15 +613,13 @@ code to be loaded.
                    (())
                  (cond ((null cdr)
                         (return (nreconc result
-                                         (car (push (gensym "LOOP-IGNORED-")
-                                                    ignores)))))
+                                         (car (push (gensym "_") ignores)))))
                        ((atom cdr)
                         (return (nreconc result cdr)))
                        ((consp (car cdr))
                         (push (list (transform (car cdr))) result))
                        ((null (car cdr))
-                        (push (car (push (gensym "LOOP-IGNORED-")
-                                         ignores))
+                        (push (car (push (gensym "_") ignores))
                               result))
                        (t
                         (push (car cdr) result))))))
@@ -769,6 +785,9 @@ code to be loaded.
                   "")))
            #+sb-unicode
            ((csubtypep ctype (specifier-type 'extended-char))
+            #+sb-xc-host
+            (error "Unimplemented on cross-compiler.")
+            #-sb-xc-host
             (code-char base-char-code-limit))
            ((csubtypep ctype (specifier-type 'character))
             #\x)
@@ -896,7 +915,7 @@ code to be loaded.
 (defun loop-make-var (name initialization dtype &optional step-var-p
                                                 &aux (loop *loop*))
   (cond ((null name)
-         (setq name (gensym "LOOP-IGNORE-"))
+         (setq name (gensym "_"))
          (push (list name (or initialization (loop-typed-init dtype step-var-p)))
                (vars loop))
          (push `(ignore ,name) (declarations loop))
@@ -917,7 +936,7 @@ code to be loaded.
                (vars loop)))
         (initialization
          (check-var-name name)
-         (let ((newvar (gensym "LOOP-DESTRUCTURE-")))
+         (let ((newvar (gensym "DS")))
            (loop-declare-var name dtype :desetq t)
            (push (list newvar initialization) (vars loop))
            ;; (DESETQ *LOOP*) gathered in reverse order.
@@ -986,10 +1005,6 @@ code to be loaded.
                                     :desetq desetq))))
         (t (error "invalid LOOP variable passed in: ~S" name))))
 
-(defun loop-maybe-bind-form (form data-type)
-  (if (constantp form)
-      form
-      (loop-make-var (gensym "LOOP-BIND-") form data-type)))
 
 (defun loop-do-if (for negatep &aux (loop *loop*) (universe (universe loop)))
   (let ((form (loop-get-form))
@@ -1064,20 +1079,6 @@ code to be loaded.
 (defun loop-do-return ()
   (loop-emit-body (loop-construct-return (loop-get-form))))
 
-;;;; value accumulation: LIST
-
-(defstruct (loop-collector
-            (:copier nil)
-            (:predicate nil))
-  name
-  class
-  (history nil)
-  (tempvars nil)
-  specified-type
-  dtype
-  (data nil)) ;collector-specific data
-(declaim (sb-ext:freeze-type loop-collector))
-
 (sb-xc:defmacro with-sum-count (lc &body body)
   (let* ((type (loop-collector-dtype lc))
          (temp-var (car (loop-collector-tempvars lc))))
@@ -1140,11 +1141,11 @@ code to be loaded.
     (let ((tempvars (loop-collector-tempvars lc)))
       (unless tempvars
         (setf (loop-collector-tempvars lc)
-              (setq tempvars (list* (gensym "LOOP-LIST-HEAD-")
-                                    (gensym "LOOP-LIST-TAIL-")
+              (setq tempvars (list* (gensym "HEAD")
+                                    (gensym "TAIL")
                                     (and (loop-collector-name lc)
                                          (list (loop-collector-name lc))))))
-        (push `(with-loop-list-collection-head ,tempvars) (wrappers *loop*))
+        (push `(with-loop-list-collection-head (,lc ,@tempvars)) (wrappers *loop*))
         (unless (loop-collector-name lc)
           (loop-emit-final-value `(loop-collect-answer ,(car tempvars)
                                                        ,@(cddr tempvars)))))
@@ -1165,7 +1166,7 @@ code to be loaded.
       (unless tempvars
         (setf (loop-collector-tempvars lc)
               (setq tempvars (list (or (loop-collector-name lc)
-                                  (gensym "LOOP-SUM-")))))
+                                       (gensym "SUM")))))
         (unless (loop-collector-name lc)
           (loop-emit-final-value (car (loop-collector-tempvars lc))))
         (push `(with-sum-count ,lc) (wrappers *loop*)))
@@ -1187,7 +1188,7 @@ code to be loaded.
         (setf (loop-collector-data lc)
               (setq data (make-loop-minimax
                           (or (loop-collector-name lc)
-                              (gensym "LOOP-MAXMIN-"))
+                              (gensym "MINMAX"))
                           (loop-collector-dtype lc))))
         (unless (loop-collector-name lc)
           (loop-emit-final-value (loop-minimax-answer-variable data)))
@@ -1235,7 +1236,7 @@ code to be loaded.
                       `(mod ,(1+ (ceiling count))))
                      (t
                       `(integer ,(ceiling count))))))
-    (let ((var (loop-make-var (gensym "LOOP-REPEAT-") `(ceiling ,form) type)))
+    (let ((var (loop-make-var (gensym "REP") `(ceiling ,form) type)))
       (push `(if (<= ,var 0) (go end-loop) (decf ,var)) (before-loop loop))
       (push `(if (<= ,var 0) (go end-loop) (decf ,var)) (after-body loop))
       ;; FIXME: What should
@@ -1343,7 +1344,7 @@ code to be loaded.
 (defun loop-when-it-var (&aux (loop *loop*))
   (or (when-it-var loop)
       (setf (when-it-var loop)
-            (loop-make-var (gensym "LOOP-IT-") nil nil))))
+            (loop-make-var (gensym "IT") nil nil))))
 
 ;;;; various FOR/AS subdispatches
 
@@ -1364,8 +1365,10 @@ code to be loaded.
 
 (defun loop-for-across (var val data-type)
   (loop-make-var var nil data-type)
-  (let ((vector-var (gensym "LOOP-ACROSS-VECTOR-"))
-        (index-var (gensym "LOOP-ACROSS-INDEX-")))
+  (let ((vector-var (gensym "V"))
+        (data-var (gensym "D"))
+        (index-var (gensym "I"))
+        (compiling (sb-c::compiling-p (macro-environment *loop*))))
     (multiple-value-bind (vector-form constantp vector-value)
         (loop-constant-fold-if-possible val 'vector)
       (loop-make-var
@@ -1373,15 +1376,24 @@ code to be loaded.
        (if (and (consp vector-form) (eq (car vector-form) 'the))
            (cadr vector-form)
            'vector))
-      (loop-make-var index-var 0 'fixnum)
+      (cond (compiling
+             (push `(multiple-value-bind (,data-var ,index-var)
+                        (%data-vector-and-index/known ,vector-var 0))
+                   (wrappers *loop*)))
+            (t
+             (loop-make-var index-var 0 'fixnum)
+             (setf data-var vector-var)))
       (let* ((length-form (if constantp
                               (length vector-value)
-                              (let ((v (gensym "LOOP-ACROSS-LIMIT-")))
-                                   (push `(let ((,v (length ,vector-var))))
+                              (let ((v (gensym "LIM")))
+                                (push (if compiling
+                                          `(let ((,v (+ (length ,vector-var) ,index-var)))
+                                             (sb-c::%in-bounds-constraint ,data-var ,v))
+                                          `(let ((,v (length ,vector-var)))))
                                          (wrappers *loop*))
                                    v)))
              (test `(>= ,index-var ,length-form))
-             (step `(,var (aref ,vector-var ,index-var)))
+             (step `(,var (aref ,data-var ,index-var)))
              (pstep `(,index-var (1+ ,index-var))))
         `(,test ,step () ,pstep)))))
 
@@ -1397,7 +1409,7 @@ code to be loaded.
           ((and (consp stepper) (eq (car stepper) 'function))
            (list (cadr stepper) listvar))
           (t
-           `(funcall ,(loop-make-var (gensym "LOOP-FN-") stepper 'function)
+           `(funcall ,(loop-make-var (gensym "F") stepper 'function)
                      ,listvar)))))
 
 (defun loop-for-on (var val data-type)
@@ -1405,15 +1417,17 @@ code to be loaded.
       (loop-constant-fold-if-possible val)
     (let ((listvar var))
       (cond ((and var (symbolp var))
-             (loop-make-var var list data-type))
+             (loop-make-var var
+                            ;; Don't want to assert the type, as ENDP will do that
+                            `(the* (list :use-annotations t :source-form ,list) ,list)
+                            data-type))
             (t
              (loop-make-var (setq listvar (gensym)) list 't)
              (loop-make-var var nil data-type)))
       (let ((list-step (loop-list-step listvar)))
         (let* ((first-endtest
-                ;; mysterious comment from original CMU CL sources:
-                ;;   the following should use `atom' instead of `endp',
-                ;;   per [bug2428]
+                ;; the following should use `atom' instead of `endp',
+                ;; per 6.1.2.1.3
                 `(atom ,listvar))
                (other-endtest first-endtest))
           (when (and constantp (listp list-value))
@@ -1430,21 +1444,45 @@ code to be loaded.
                               `(,first-endtest ,step () ,pseudo)))))))))))
 
 (defun loop-for-in (var val data-type)
-  (multiple-value-bind (list constantp list-value)
-      (loop-constant-fold-if-possible val)
-    (let ((listvar (gensym "LOOP-LIST-")))
-      (loop-make-var var nil data-type)
-      (loop-make-var listvar list t)
-      (let ((list-step (loop-list-step listvar)))
-        (let* ((first-endtest `(endp ,listvar))
-               (other-endtest first-endtest)
-               (step `(,var (car ,listvar)))
-               (pseudo-step `(,listvar ,list-step)))
-          (when (and constantp (listp list-value))
-            (setq first-endtest (null list-value)))
-          `(,other-endtest ,step () ,pseudo-step
-            ,@(and (neq first-endtest other-endtest)
-                   `(,first-endtest ,step () ,pseudo-step))))))))
+  (cond ((and (typep val '(cons (eql reverse) (cons t null)))
+              (not (sb-c::fun-lexically-notinline-p 'reverse
+                                                    (macro-environment *loop*)))
+              (let ((stepper (and (loop-tequal (car (source-code *loop*)) :by)
+                                  (source-code *loop*))))
+                (cond ((member (cadr stepper) '(#'cddr 'cddr) :test #'equal)
+                       (loop-pop-source)
+                       (loop-pop-source)
+                       (loop-for-across var `(list-reverse-into-vector-cddr ,(second val)) data-type))
+                      ((not stepper)
+                       (loop-for-across var `(list-reverse-into-vector ,(second val)) data-type))))))
+        ((and (typep val '(cons (eql sort) (cons (cons (eql copy-list) (cons t null)) cons)))
+              (not (sb-c::fun-lexically-notinline-p 'copy-list
+                                                    (macro-environment *loop*)))
+              (let ((stepper (and (loop-tequal (car (source-code *loop*)) :by)
+                                  (source-code *loop*))))
+                (cond ((not stepper)
+                       (destructuring-bind (sort (copy-list list) &rest args) val
+                         (declare (ignore sort copy-list))
+                         (loop-for-across var `(sort (coerce (the list ,list) 'vector) ,@args) data-type)))))))
+        (t
+         (multiple-value-bind (list constantp list-value)
+             (loop-constant-fold-if-possible val)
+           (let ((listvar (gensym "L")))
+             (loop-make-var var nil data-type)
+             (loop-make-var listvar
+                            ;; Don't want to assert the type, as ENDP will do that
+                            `(the* (list :use-annotations t :source-form ,list) ,list)
+                            t)
+             (let ((list-step (loop-list-step listvar)))
+               (let* ((first-endtest `(endp ,listvar))
+                      (other-endtest first-endtest)
+                      (step `(,var (car ,listvar)))
+                      (pseudo-step `(,listvar ,list-step)))
+                 (when (and constantp (listp list-value))
+                   (setq first-endtest (null list-value)))
+                 `(,other-endtest ,step () ,pseudo-step
+                                  ,@(and (neq first-endtest other-endtest)
+                                         `(,first-endtest ,step () ,pseudo-step))))))))))
 
 ;;;; iteration paths
 
@@ -1641,14 +1679,14 @@ code to be loaded.
             (setq endform (if limit-constantp
                               `',limit-value
                               (loop-make-var
-                                 (gensym "LOOP-LIMIT-") form
+                                 (gensym "LIM") form
                                  `(and ,indexv-type real)))))
            (:by
             (multiple-value-setq (form stepby-constantp stepby)
               (loop-constant-fold-if-possible form
                                               `(and ,indexv-type (real (0)))))
             (unless stepby-constantp
-              (loop-make-var (setq stepby (gensym "LOOP-STEP-BY-"))
+              (loop-make-var (setq stepby (gensym "STEP"))
                  form
                  `(and ,indexv-type (real (0)))
                  t)))
@@ -1693,7 +1731,7 @@ code to be loaded.
        (cond ((member dir '(nil :up))
               (when (or limit-given default-top)
                 (unless limit-given
-                  (loop-make-var (setq endform (gensym "LOOP-SEQ-LIMIT-"))
+                  (loop-make-var (setq endform (gensym "LIM"))
                      nil
                      indexv-type)
                   (push `(setq ,endform ,default-top) (prologue loop)))
@@ -1774,8 +1812,8 @@ code to be loaded.
          (loop-error "Too many prepositions!"))
         ((null prep-phrases)
          (loop-error "missing OF or IN in ~S iteration path")))
-  (let ((ht-var (gensym "LOOP-HASHTAB-"))
-        (next-fn (gensym "LOOP-HASHTAB-NEXT-"))
+  (let ((ht-var (gensym "HT")) ; the table
+        (next-fn (gensym "NEXT")) ; WITH-HASH-TABLE-ITERATOR name
         (dummy-predicate-var nil)
         (post-steps nil))
     (multiple-value-bind (other-var other-p)
@@ -1791,7 +1829,7 @@ code to be loaded.
             dummy-predicate-var (loop-when-it-var))
       (let* ((key-var nil)
              (val-var nil)
-             (variable (or variable (gensym "LOOP-HASH-VAR-TEMP-")))
+             (variable (or variable (gensym "VAR")))
              (bindings `((,variable nil ,data-type)
                          (,ht-var ,(cadar prep-phrases))
                          ,@(and other-p other-var `((,other-var nil))))))
@@ -1803,12 +1841,12 @@ code to be loaded.
         (push `(with-hash-table-iterator (,next-fn ,ht-var)) (wrappers *loop*))
         (when (or (consp key-var) data-type)
           (setq post-steps
-                `(,key-var ,(setq key-var (gensym "LOOP-HASH-KEY-TEMP-"))
+                `(,key-var ,(setq key-var (gensym "KTMP"))
                            ,@post-steps))
           (push `(,key-var nil) bindings))
         (when (or (consp val-var) data-type)
           (setq post-steps
-                `(,val-var ,(setq val-var (gensym "LOOP-HASH-VAL-TEMP-"))
+                `(,val-var ,(setq val-var (gensym "VTMP"))
                            ,@post-steps))
           (push `(,val-var nil) bindings))
         `(,bindings                     ;bindings
@@ -1827,9 +1865,9 @@ code to be loaded.
          (bug "Unknown preposition ~S." (caar prep-phrases))))
   (unless (symbolp variable)
     (loop-error "Destructuring is not valid for package symbol iteration."))
-  (let ((pkg-var (gensym "LOOP-PKGSYM-"))
-        (next-fn (gensym "LOOP-PKGSYM-NEXT-"))
-        (variable (or variable (gensym "LOOP-PKGSYM-VAR-")))
+  (let ((pkg-var (gensym "SYM"))
+        (next-fn (gensym "NEXT")) ; WITH-PACKAGE-ITERATOR name
+        (variable (or variable (gensym "VAR")))
         (package (or (cadar prep-phrases) '*package*)))
     (push `(with-package-iterator (,next-fn ,pkg-var ,@symbol-types))
           (wrappers *loop*))
