@@ -110,10 +110,9 @@
      (truly-the single-float (imagpart number)))
     ((complex rational)
      (%imagpart number))
-    (float
-     (* 0 number))
-    (number
-     0)))
+    (single-float 0f0)
+    (double-float 0d0)
+    (t 0)))
 
 (defun conjugate (number)
   "Return the complex conjugate of NUMBER. For non-complex numbers, this is
@@ -1707,39 +1706,42 @@ and the number of 0 bits if INTEGER is negative."
 (declaim (maybe-inline fixnum-gcd))
 #-arm
 (defun fixnum-gcd (u v)
-  (declare (optimize (safety 0) speed)
-           (fixnum u v))
-  (let ((u (abs u))
-        (v (abs v)))
-    (declare (sb-vm:signed-word u v))
-    (loop (setf (values u v)
-                (values v (rem u (the (not (eql 0)) v))))
-          (if (zerop v)
-              (return u)))))
+  (cond ((eql u 0) (abs v))
+        ((eql v 0) (abs u))
+        (t
+         (let ((u (abs u))
+               (v (abs v)))
+           (declare (sb-vm:signed-word u v))
+           (loop (setf (values u v)
+                       (values v (rem u (the (not (eql 0)) v))))
+                 (if (zerop v)
+                     (return (truly-the (integer 0 #.(1+ most-positive-fixnum)) u))))))))
 
 ;;; But 32-bit arm has no division instruction
 #+arm
 (defun fixnum-gcd (u v)
-  (declare (optimize (safety 0)))
   (locally
       (declare (optimize (speed 3) (safety 0)))
-    (do ((k 0 (1+ k))
-         (u (abs u) (ash u -1))
-         (v (abs v) (ash v -1)))
-        ((oddp (logior u v))
-         (do ((temp (if (oddp u) (- v) (ash u -1))
-                    (ash temp -1)))
-             (nil)
-           (declare (fixnum temp))
-           (when (oddp temp)
-             (if (plusp temp)
-                 (setq u temp)
-                 (setq v (- temp)))
-             (setq temp (- u v))
-             (when (zerop temp)
-               (return (the (integer 0 #.(1+ most-positive-fixnum)) (ash u k)))))))
-      (declare (type (mod #.sb-vm:n-word-bits) k)
-               (type sb-vm:signed-word u v)))))
+    (cond ((eql u 0) (abs v))
+          ((eql v 0) (abs u))
+          (t
+           (do ((k 0 (1+ k))
+                (u (abs u) (ash u -1))
+                (v (abs v) (ash v -1)))
+               ((oddp (logior u v))
+                (do ((temp (if (oddp u) (- v) (ash u -1))
+                           (ash temp -1)))
+                    (nil)
+                  (declare (fixnum temp))
+                  (when (oddp temp)
+                    (if (plusp temp)
+                        (setq u temp)
+                        (setq v (- temp)))
+                    (setq temp (- u v))
+                    (when (zerop temp)
+                      (return (the (integer 0 #.(1+ most-positive-fixnum)) (ash u k)))))))
+             (declare (type (mod #.sb-vm:n-word-bits) k)
+                      (type sb-vm:signed-word u v)))))))
 
 ;;; Do the GCD of two integer arguments.
 ;;; We pick off the special case
@@ -1809,8 +1811,7 @@ and the number of 0 bits if INTEGER is negative."
   (declare (explicit-check))
   (number-dispatch ((x number) (y number))
     (float-contagion / x y (ratio integer))
-
-    ((complex complex)
+    (((foreach integer ratio single-float double-float complex) complex)
      (let* ((rx (realpart x))
             (ix (imagpart x))
             (ry (realpart y))
@@ -1824,18 +1825,6 @@ and the number of 0 bits if INTEGER is negative."
                   (dn (+ iy (* r ry))))
              (complex (/ (+ (* rx r) ix) dn)
                       (/ (- (* ix r) rx) dn))))))
-    (((foreach integer ratio single-float double-float) complex)
-     (let* ((ry (realpart y))
-            (iy (imagpart y)))
-       (if (> (abs ry) (abs iy))
-           (let* ((r (/ iy ry))
-                  (dn (+ ry (* r iy))))
-             (complex (/ x dn)
-                      (/ (- (* x r)) dn)))
-           (let* ((r (/ ry iy))
-                  (dn (+ iy (* r ry))))
-             (complex (/ (* x r) dn)
-                      (/ (- x) dn))))))
     ((complex (or rational float))
      (canonical-complex (/ (realpart x) y)
                         (/ (imagpart x) y)))
