@@ -103,6 +103,7 @@
                                          (compression nil)
                                          #+win32
                                          (application-type :console))
+  #.(format nil
   "Save a \"core image\", i.e. enough information to restart a Lisp
 process later in the same state, in the file of the specified name.
 Only global state is preserved: the stack is unwound in the process.
@@ -122,7 +123,7 @@ The following &KEY arguments are defined:
      If true, arrange to combine the SBCL runtime and the core image
      to create a standalone executable.  If false (the default), the
      core image will not be executable on its own. Executable images
-     always behave as if they were passed the --noinform runtime option.
+     always behave as if they were passed the --noinform runtime option.~A
 
   :SAVE-RUNTIME-OPTIONS
      If true, values of runtime options --dynamic-space-size and
@@ -204,6 +205,10 @@ This implementation is not as polished and painless as you might like:
 This isn't because we like it this way, but just because there don't
 seem to be good quick fixes for either limitation and no one has been
 sufficiently motivated to do lengthy fixes."
+  #+elf "
+     If :EXECUTABLE is :ELF-OBJECT, then the resulting core will be
+     wrapped in a .o which requires further linking. (EXPERIMENTAL)"
+  #-elf "")
   (declare (ignore environment-name))
   (declare (ignorable root-structures))
   (when (and callable-exports toplevel-supplied)
@@ -256,7 +261,8 @@ sufficiently motivated to do lengthy fixes."
           ;; since the GC will invalidate the stack.
           (sb-kernel::unsafe-clear-roots sb-vm:+highest-normal-generation+)
           (gc-and-save name
-                       (foreign-bool executable)
+                       #+elf (if (eq executable :elf-object) 2 (foreign-bool executable))
+                       #-elf (foreign-bool executable)
                        (foreign-bool purify)
                        (case save-runtime-options
                          (:accept-runtime-options 2)
@@ -318,20 +324,34 @@ sufficiently motivated to do lengthy fixes."
   #+sb-thread
   (let (error)
     (with-system-mutex (sb-thread::*make-thread-lock*)
-      (finalizer-thread-stop)
       (sb-thread::%dispose-thread-structs)
-      (let ((threads (sb-thread:list-all-threads))
-            (starting
-             (setq sb-thread::*starting-threads* ; ordinarily pruned in MAKE-THREAD
-                   (delete 0 sb-thread::*starting-threads*)))
-            (joinable sb-thread::*joinable-threads*))
-        (when (or (cdr threads) starting joinable)
-          (let* ((interactive (sb-thread::interactive-threads))
-                 (other (union (set-difference threads interactive)
-                               (union starting joinable))))
-            (setf error (make-condition 'save-with-multiple-threads-error
-                                        :interactive-threads interactive
-                                        :other-threads other))))))
+      ;; As a consequence of a discovery I made about a nasty but common pattern
+      ;; among users of doing (mapc #'some-harmful-operation (list-all-threads),
+      ;; the finalizer thread is not present in (LIST-ALL-THREADS). Better to pretend
+      ;; it doesn't exist than let users affect it (backtrace/terminate/whatever).
+      ;; The same is not true of the signal-waiter thread on #+(and unix sb-safepoint).
+      ;; Maybe it should be. But that being the case, we need to remove the sigwait
+      ;; thread from list-all-threads.
+      (let* ((userthreads
+              (remove-if #'sb-thread::thread-ephemeral-p
+                         (sb-thread:list-all-threads)))
+             (starting-userthreads
+              ;; Cleanup of *starting-threads* is normally done in START-THREAD guarded
+              ;; by *make-thread-lock*. We need to avoid racing with that.
+              (mapcan (lambda (x)
+                        (if (and (sb-thread::thread-p x)
+                                 (not (sb-thread::thread-ephemeral-p x)))
+                            (list x)))
+                      sb-thread::*starting-threads*))
+             (joinable sb-thread::*joinable-threads*))
+        (if (or (cdr userthreads) starting-userthreads joinable)
+            (let* ((interactive (sb-thread::interactive-threads))
+                   (other (union (set-difference userthreads interactive)
+                                 (union starting-userthreads joinable))))
+              (setf error (make-condition 'save-with-multiple-threads-error
+                                          :interactive-threads interactive
+                                          :other-threads other)))
+            (finalizer-thread-stop))))
     (when error (error error))
     #+allocator-metrics (setq sb-thread::*allocator-metrics* nil)
     (setq sb-thread::*sprof-data* nil))
