@@ -33,14 +33,16 @@
 (defun numeric-contagion-constraint-back (x y gen constraint consequent alternative &key
                                                                                       complex-p
                                                                                       integer
-                                                                                      (same-leaf-not-complex t)
+                                                                                      same-leaf-not-complex
                                                                                       (x-type (lvar-type x))
                                                                                       (y-type (lvar-type y)))
   (flet ((add (lvar type)
            (add-back-constraint gen 'typep lvar type consequent))
          (add-alt (lvar type)
            (add-back-constraint gen 'typep lvar type alternative)))
-    (let ((real-type (if complex-p ;; complex rationals multiplied by 0 will produce an integer 0.
+    (let ((real-type (if complex-p
+                         ;; (* 0 complex-rational) => rational
+                         ;; (/ 0 complex-rational) => 0
                          (specifier-type '(and real (not (eql 0))))
                          (specifier-type 'real))))
       (cond ((csubtypep constraint (specifier-type 'rational))
@@ -159,8 +161,8 @@
              (add y (specifier-type 'complex)))
             ((and x-type
                   (csubtypep constraint (specifier-type 'real)))
-             (let ((x-realp (csubtypep x-type (specifier-type 'real)))
-                   (y-realp (csubtypep y-type (specifier-type 'real))))
+             (let ((x-realp (csubtypep x-type real-type))
+                   (y-realp (csubtypep y-type real-type)))
                (when (and alternative
                           (eq constraint (specifier-type 'real)))
                  (cond (x-realp
@@ -203,10 +205,12 @@
                            (int interval y x)
                            (int interval x y))
                          t))))
-              (numeric-contagion-constraint-back x y gen constraint nil alternative :integer t))
+              (numeric-contagion-constraint-back x y gen constraint nil alternative :integer t
+                                                                                    :same-leaf-not-complex t))
              (t
               (numeric-contagion-constraint-back x y gen constraint consequent alternative
-                                                 :integer t)))))))
+                                                 :integer t
+                                                 :same-leaf-not-complex t)))))))
 
 (defun -constraint-propagate-back (x y x-type y-type kind constraint gen consequent alternative)
   (case kind
@@ -253,8 +257,8 @@
     (typep
      (flet ((add (lvar type)
               (add-back-constraint gen 'typep lvar type consequent)))
-       (let ((complex-p (or (types-equal-or-intersect (lvar-type x) (specifier-type 'complex))
-                            (types-equal-or-intersect (lvar-type y) (specifier-type 'complex)))))
+       (let ((complex-p (or (lvar-intersectp x complex)
+                            (lvar-intersectp y complex))))
          (cond ((and
                  (csubtypep constraint (specifier-type 'integer))
                  (let* ((rational-type (if complex-p
@@ -292,7 +296,9 @@
                                      (int interval x y)
                                      t))))
                            ((same-leaf-ref-p x y)
-                            (add x (specifier-type 'integer)))))))
+                            (add x (if (csubtypep constraint (specifier-type 'unsigned-byte))
+                                       (specifier-type 'integer)
+                                       (specifier-type '(or integer (complex rational))))))))))
                 (numeric-contagion-constraint-back x y gen constraint nil alternative :integer t))
                (t
                 (numeric-contagion-constraint-back x y gen constraint consequent alternative :complex-p complex-p
@@ -302,7 +308,8 @@
   (declare (ignore nth-value))
   (case kind
     (typep
-     (numeric-contagion-constraint-back x y gen constraint consequent alternative :same-leaf-not-complex nil))))
+     (numeric-contagion-constraint-back x y gen constraint consequent alternative
+                                        :complex-p (lvar-intersectp y complex)))))
 
 (defoptimizers constraint-propagate-back (car cdr) ((x) node nth-value kind constraint gen consequent alternative)
   (declare (ignore nth-value alternative))
