@@ -83,10 +83,9 @@
                    ;; #+immobile-space implies a relocatable alien linkage space. And x86-64 always
                    ;; has relocatable linkage tables
                    #-(or x86-64 immobile-space) (alien-linkage ,alien-linkage-space-size)
-                   ;; safepoint on 64-bit uses a relocatable trap page just below the card mark
-                   ;; table, which works nicely assuming a register is wired to the card table
+                   ;; x86-64 uses a relocatable trap page just below the card mark
+                   ;; table (wired to a register). Other platforms allocate a separate page.
                    #+(and sb-safepoint (not x86-64))
-                   ;; Must be just before NIL.
                    (safepoint ,(symbol-value '+backend-page-bytes+))
                    (static ,small-space-size)
                    #+darwin-jit (static-code ,small-space-size)))
@@ -237,10 +236,6 @@
     #+(and x86-64 sb-thread (not gs-seg))
     sb-aprof::*n-profile-sites*
 
-    ;; runtime linking of lisp->C calls (regardless of whether
-    ;; the C function is in a dynamic shared object or not)
-    +required-foreign-symbols+
-
     ;;; The following symbols aren't strictly required to be static
     ;;; - they are not accessed from C - but we make them static in order
     ;;; to (perhaps) micro-optimize access in Lisp.
