@@ -787,6 +787,7 @@
                                    target-block))
                  (cmp (ir2-block-start-vop target-block)))
             (when (and cmp
+                       (eq cmp (ir2-block-last-vop target-block))
                        (singleton-p (ir2block-predecessors target-block)))
               (let ((integer (vop-args vop))
                     (args (vop-args cmp))
@@ -1085,6 +1086,8 @@
          (not (tn-ref-next writes))
          (neq (tn-vertex tn) :alias))))
 
+;;; Broken by mask-signed-field using move-from-word/fixnum to sign-extend words
+#+()
 (defoptimizer (vop-optimize sb-vm::move-from-word/fixnum)
     (vop)
   (vop-bind (in) (out) vop
@@ -1189,7 +1192,7 @@
                                  (let ((vop2 (branch-destination vop nil)))
                                    (and vop2
                                         (eq (vop-name vop2) not-vop)
-                                        (singleton-p (ir2block-predecessors (vop-block dest)))
+                                        (singleton-p (ir2block-predecessors (vop-block vop2)))
                                         (let ((dest2 (branch-destination vop2)))
                                           (when (and dest2
                                                      (singleton-p (ir2block-predecessors (vop-block dest2)))
@@ -1200,33 +1203,33 @@
                                                            (eq in22 in))
                                                   (let ((dest-tn1 (make-mov-vop out2 dest))
                                                         (dest-tn2 (make-mov-vop out22 dest2)))
-                                                    (emit-and-insert-vop
-                                                     (vop-node vop) (vop-block vop)
-                                                     (template-or-lose 'sb-vm::un/signed-byte-64-p-move-to-word)
-                                                     (reference-tn-refs (vop-args vop) nil)
-                                                     (reference-tn-list (list dest-tn1 dest-tn2) t)
-                                                     vop
-                                                     (append (vop-codegen-info vop)
-                                                             (vop-codegen-info vop2)
-                                                             (list (vop-label (branch-destination vop2 nil))))))
-                                                  (delete-vop vop)
-                                                  (delete-vop vop2)
-                                                  (delete-vop dest)
-                                                  (delete-vop dest2)
-                                                  t)))))))))
+                                                    (prog1
+                                                        (emit-and-insert-vop
+                                                         (vop-node vop) (vop-block vop)
+                                                         (template-or-lose 'sb-vm::un/signed-byte-64-p-move-to-word)
+                                                         (reference-tn-refs (vop-args vop) nil)
+                                                         (reference-tn-list (list dest-tn1 dest-tn2) t)
+                                                         vop
+                                                         (append (vop-codegen-info vop)
+                                                                 (vop-codegen-info vop2)
+                                                                 (list (vop-label (branch-destination vop2 nil)))))
+                                                      (delete-vop vop)
+                                                      (delete-vop vop2)
+                                                      (delete-vop dest)
+                                                      (delete-vop dest2))))))))))))
                            (t
                             (let ((dest-tn (make-mov-vop out2 dest)))
-                              (emit-and-insert-vop
-                               (vop-node vop) (vop-block vop)
-                               (template-or-lose new-vop)
-                               (reference-tn-refs (vop-args vop) nil)
-                               (reference-tn dest-tn t)
-                               vop (vop-codegen-info (or branch-if vop)))
-                              (delete-vop vop)
-                              (when branch-if
-                                (delete-vop branch-if))
-                              (delete-vop dest)))))))))
-           nil))
+                              (prog1
+                                  (emit-and-insert-vop
+                                   (vop-node vop) (vop-block vop)
+                                   (template-or-lose new-vop)
+                                   (reference-tn-refs (vop-args vop) nil)
+                                   (reference-tn dest-tn t)
+                                   vop (vop-codegen-info (or branch-if vop)))
+                                (delete-vop vop)
+                                (when branch-if
+                                  (delete-vop branch-if))
+                                (delete-vop dest))))))))))))
 
   (when-vop-existsp (:named sb-vm::signed-byte-64-p-move-to-word)
     (defoptimizer (vop-optimize signed-byte-64-p select-representations) (vop)
