@@ -44,7 +44,9 @@
 (defun make-return-pc-save-location (env)
   (let ((ptype *fixnum-primitive-type*))
     (specify-save-tn
-     (environment-debug-live-tn (make-normal-tn ptype) env)
+     (environment-debug-live-tn
+      (make-wired-tn *fixnum-primitive-type* any-reg-sc-number ra-offset)
+      env)
      (make-wired-tn ptype control-stack-arg-scn ra-save-offset))))
 
 ;;; Make a TN for the standard argument count passing location.  We
@@ -139,7 +141,6 @@
     (move res csp-tn)
     (let ((size (* (max 1 (sb-allocated-size 'control-stack))
                    n-word-bytes)))
-      (storew cfp-tn res ocfp-save-offset)
       (add-imm csp-tn csp-tn size 'allocate-frame tmp))
     ;; number stack frame
     (when (ir2-environment-number-stack-p callee)
@@ -161,13 +162,9 @@
   (:temporary (:sc unsigned-reg
                :unused-if (typep (* nargs n-word-bytes) 'short-immediate)) tmp)
   (:generator 2
-    (if (<= nargs register-arg-count)
-        (storew cfp-tn csp-tn ocfp-save-offset)
-        (let ((size (* nargs n-word-bytes)))
-          (move res csp-tn)
-          (storew cfp-tn res ocfp-save-offset)
-          ;; CSP += size
-          (add-imm csp-tn csp-tn size 'allocate-full-call-frame tmp)))))
+    (when (> nargs register-arg-count)
+      (move res csp-tn)
+      (add-imm csp-tn csp-tn (* nargs n-word-bytes) 'allocate-full-call-frame tmp))))
 
 
 
@@ -532,11 +529,6 @@
                     :to :eval)
                    return-pc-pass)
 
-        (:temporary (:sc any-reg
-             :offset t7-offset
-             :to :eval)
-            tmp)
-
        ,@(unless (eq named :direct)
          `((:temporary (:sc descriptor-reg :offset lexenv-offset
                         :from (:argument ,(if (eq return :tail) 0 1))
@@ -562,10 +554,9 @@
            '((:temporary (:scs (descriptor-reg) :from :eval) move-temp)))
 
        (:temporary (:scs (descriptor-reg) :to :eval) stepping)
-
+       (:temporary (:scs (interior-reg)) lip)
        ,@(unless (eq return :tail)
-           '((:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)
-             (:temporary (:scs (interior-reg)) lip)))
+           '((:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)))
 
        (:generator ,(+ (if named 5 0)
                        (if variable 19 1)
@@ -685,10 +676,8 @@
                    (do-next-filler)
                    (return)))
              (note-this-location vop :call-site)
-             (inst addi.d tmp function (- fun-pointer-lowtag))
-             (inst jirl ,(if (eq return :tail) 'zero-tn 'ra-tn)
-                          tmp
-                          (ash simple-fun-insts-offset word-shift)))
+             (inst addi.d lip function (- (ash simple-fun-insts-offset word-shift) fun-pointer-lowtag))
+             (inst jirl ,(if (eq return :tail) 'zero-tn 'ra-tn) lip 0))
            ,@(ecase return
                (:fixed
                 '((emit-label label)
