@@ -385,6 +385,22 @@
                                      #',transform-name ,important))
                    names)))))
 
+
+(defun make-optimizer-name (name)
+  (flet ((function-name (name)
+           (etypecase name
+             (symbol name)
+             ((cons (eql setf) (cons symbol null))
+              (symbolicate (car name) "-" (cadr name))))))
+    (if (symbolp name)
+        name
+        (symbolicate (function-name (first name))
+                     "-"
+                     (if (consp (second name))
+                         (caadr name)
+                         (second name))
+                     "-OPTIMIZER"))))
+
 ;;; Create a function which parses combination args according to WHAT
 ;;; and LAMBDA-LIST, where WHAT is either a function name or a list
 ;;; (FUN-NAME KIND) and does some KIND of optimization.
@@ -410,19 +426,7 @@
                               &optional (node (gensym))
                               &rest vars)
                         &body body)
-  (let ((name (flet ((function-name (name)
-                       (etypecase name
-                         (symbol name)
-                         ((cons (eql setf) (cons symbol null))
-                          (symbolicate (car name) "-" (cadr name))))))
-                (if (symbolp what)
-                    what
-                    (symbolicate (function-name (first what))
-                                 "-"
-                                 (if (consp (second what))
-                                     (caadr what)
-                                     (second what))
-                                 "-OPTIMIZER")))))
+  (let ((name (make-optimizer-name what)))
     (cond ((typep what '(cons (eql vop-optimize)))
            `(progn
               (defun ,name (,lambda-list)
@@ -435,7 +439,7 @@
                                                `#',name)))))
           ((typep what '(cons t (cons (eql fold-p))))
            `(progn
-              (defun ,name (,lambda-list)
+              (defun ,name ,lambda-list
                 ,@body)
               (setf (,(let ((*package* (sb-xc:symbol-package 'fun-info)))
                         (symbolicate "FUN-INFO-" (second what)))
@@ -471,8 +475,7 @@
                                                 collect bind)))
                     ,@more-decls ,@forms))
                 ,@(when (consp what)
-                    `((setf (,(let ((*package* (sb-xc:symbol-package 'fun-info)))
-                                (symbolicate "FUN-INFO-" (second what)))
+                    `((setf (,(package-symbolicate #.(find-package "SB-C") "FUN-INFO-" (second what))
                              (fun-info-or-lose ',(first what)))
                             #',name)))))))))
 
@@ -480,17 +483,14 @@
                                      &optional (node (gensym))
                                      &rest vars)
                          &body body)
-  (let ((optimizer-name (symbolicate (car names)
-                                     "-"
-                                     kind
-                                     "-OPTIMIZER")))
+  (let* ((name (list (car names) kind))
+         (optimizer-name (make-optimizer-name name)))
     `(progn
-       (defoptimizer ,optimizer-name
+       (defoptimizer ,name
            (,lambda-list ,node ,@vars)
          ,@body)
-       ,@(loop for name in names
-               collect `(setf (,(let ((*package* (sb-xc:symbol-package 'fun-info)))
-                                  (symbolicate "FUN-INFO-" kind))
+       ,@(loop for name in (cdr names)
+               collect `(setf (,(package-symbolicate #.(find-package "SB-C") "FUN-INFO-" kind)
                                (fun-info-or-lose ',name))
                               #',optimizer-name)))))
 
@@ -655,7 +655,6 @@
            (*inline-expansions* (if (basic-combination-p ,node)
                                     (basic-combination-inline-expansions ,node)
                                     *inline-expansions*)))
-       (aver-live-component *current-component*)
        ,@forms)))
 
 ;;; *SOURCE-PATHS* is a hashtable from source code forms to the path
@@ -1022,65 +1021,3 @@ specify bindings for printer control variables.")
         (nreverse (mapcar #'car *compiler-print-variable-alist*))
         (nreverse (mapcar #'cdr *compiler-print-variable-alist*))
       ,@forms)))
-
-(defmacro make-defs (vars &body body)
-  (labels ((subst-if-with (test tree)
-             (labels ((s (subtree)
-                        (multiple-value-bind (new replace)
-                            (funcall test subtree)
-                          (cond (replace new)
-                                ((comma-p subtree)
-                                 (let ((new (s (comma-expr subtree))))
-                                   (if (eq subtree new)
-                                       subtree
-                                       (unquote new (comma-kind subtree)))))
-                                ((atom subtree) subtree)
-                                (t (let ((car (s (car subtree)))
-                                         (cdr (s (cdr subtree))))
-                                     (if (and (eq car (car subtree))
-                                              (eq cdr (cdr subtree)))
-                                         subtree
-                                         (cond ((and (typep car '(cons symbol))
-                                                     (string= (car car) "$WHEN"))
-                                                (if (eval (second car))
-                                                    (append (cddr car) cdr)
-                                                    cdr))
-                                               ((and (typep car '(cons symbol))
-                                                     (string= (car car) "$UNLESS"))
-                                                (if (eval (second car))
-                                                    cdr
-                                                    (append (cddr car) cdr)))
-                                               ((and (typep car '(cons symbol))
-                                                     (string= (car car) "$IF"))
-                                                (cons
-                                                 (if (eval (second car))
-                                                     (third car)
-                                                     (fourth car))
-                                                 cdr))
-                                               (t
-                                                (cons car cdr))))))))))
-               (s tree)))
-           (test (pattern with)
-             (lambda (x)
-               (when (symbolp x)
-                 (let* ((str (string x))
-                        (start (search pattern str)))
-                   (when start
-                     (if (equal str pattern)
-                         (values with t)
-                         (values (symbolicate (subseq str 0 start)
-                                              with
-                                              (subseq str (+ start (length pattern))))
-                                 t)))))))
-           (gen (vars body)
-             (if vars
-                 (loop for with in (cdar vars)
-                       append
-                       (gen (cdr vars)
-                            (let ((body body))
-                              (loop for v in (ensure-list (caar vars))
-                                    for w in (ensure-list with)
-                                    do (setf body (subst-if-with (test (string v) w) body)))
-                              body)))
-                 body)))
-    `(progn ,@(gen vars body))))
