@@ -84,7 +84,9 @@ open_binary(char *filename, int mode)
     return open(filename, mode);
 }
 
-#if defined LISP_FEATURE_LINUX && (defined LISP_FEATURE_X86_64 || defined LISP_FEATURE_ARM64)
+#if defined LISP_FEATURE_LINUX && \
+ (defined LISP_FEATURE_X86_64 || defined LISP_FEATURE_ARM64 \
+ || (defined LISP_FEATURE_PPC64 && defined LISP_FEATURE_LITTLE_ENDIAN))
 #define ELFCORE 1
 #elif !defined(ELFCORE)
 #define ELFCORE 0
@@ -853,7 +855,9 @@ process_directory(int count, struct ndir_entry *entry,
                 break;
 #endif
             case IMMOBILE_TEXT_CORE_SPACE_ID:
+#ifndef LISP_FEATURE_ARM64
                 TEXT_SPACE_START = addr;
+#endif
                 break;
             case DYNAMIC_CORE_SPACE_ID:
                 {
@@ -1007,7 +1011,7 @@ static bool compute_card_table_size(int saved_card_mask_nbits)
     // The card table size is a power of 2 at *least* as large
     // as the number of cards. These are the default values.
     int nbits = 13;
-    long num_gc_cards = 1L << nbits;
+    sword_t num_gc_cards = (sword_t)1 << nbits;
 
     // Sure there's a fancier way to round up to a power-of-2
     // but this is executed exactly once, so KISS.
@@ -1033,7 +1037,7 @@ static bool compute_card_table_size(int saved_card_mask_nbits)
     // Regardless of the mask implied by space size, it has to be gc_card_table_nbits wide
     // even if that is excessive - when the core is restarted using a _smaller_ dynamic space
     // size than saved at - otherwise lisp could overrun the mark table.
-    num_gc_cards = 1L << gc_card_table_nbits;
+    num_gc_cards = (sword_t)1 << gc_card_table_nbits;
 
     gc_card_table_mask =  num_gc_cards - 1;
     return patch_card_index_mask_fixups;
@@ -1387,7 +1391,7 @@ init_coreparse_spaces(int n, struct coreparse_space* input)
  * 0: No
  * -1: default, yes for compressed cores, no otherwise.
  */
-lispobj
+struct initfunctions
 load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
 {
     void *header;
@@ -1395,7 +1399,7 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
     os_vm_size_t len, remaining_len, stringlen;
     int fd = open_binary(file, O_RDONLY);
     ssize_t count;
-    lispobj initial_function = NIL;
+    struct initfunctions initfun = {0,0,0};
     struct heap_adjust adj;
     memset(&adj, 0, sizeof adj);
     sword_t linkage_table_data_page = -1;
@@ -1451,7 +1455,7 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
     // The initializer ensures that indexing into spaces[] is insensitive
     // to the space numbering and the order listed in defined_spaces.
     struct coreparse_space* spaces =
-      init_coreparse_spaces(sizeof defined_spaces/sizeof (struct coreparse_space),
+      init_coreparse_spaces(sizeof(defined_spaces)/sizeof(struct coreparse_space),
                             defined_spaces);
     bool patch_card_marking_instructions = 0;
 
@@ -1532,7 +1536,9 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
                                   spaces, &adj, patch_card_marking_instructions);
             break;
         case INITIAL_FUN_CORE_ENTRY_TYPE_CODE:
-            initial_function = adjust_word(&adj, (lispobj)*ptr);
+            memcpy(&initfun, ptr, sizeof initfun);
+            initfun.c_linkage_vector = adjust_word(&adj, initfun.c_linkage_vector);
+            initfun.lispfun = adjust_word(&adj, initfun.lispfun);
             break;
         case END_CORE_ENTRY_TYPE_CODE:
             free(header);
@@ -1546,15 +1552,15 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
             SYMBOL(FREE_TLS_INDEX)->value = sizeof (struct thread);
 #endif
             // simple-fun implies cold-init, not a warm core (it would be a closure then)
-            if (widetag_of(native_pointer(initial_function)) == SIMPLE_FUN_WIDETAG
+            if (widetag_of(native_pointer(initfun.lispfun)) == SIMPLE_FUN_WIDETAG
                 && !lisp_startup_options.noinform) {
                 fprintf(stderr, "Initial page table:\n");
                 extern void print_generation_stats(void);
                 print_generation_stats();
             }
-            sanity_check_loaded_core(initial_function);
+            sanity_check_loaded_core(initfun.lispfun);
             free(spaces);
-            return initial_function;
+            return initfun;
         case RUNTIME_OPTIONS_MAGIC: break; // already processed
         default:
             lose("unknown core header entry: %"OBJ_FMTX, (lispobj)val);
