@@ -13,6 +13,7 @@
 #define _INCLUDE_INTERRUPT_H_
 
 #include "runtime.h"
+#include <signal.h> // for NSIG
 #include <stdbool.h>
 #include <string.h>
 #include "genesis/static-symbols.h"
@@ -101,6 +102,7 @@ typedef lispobj (*call_into_lisp_lookalike)(
 extern bool interrupt_handler_pending_p(void);
 extern void interrupt_init(void);
 extern void fake_foreign_function_call(os_context_t* context);
+extern void fake_foreign_function_call_noassert(os_context_t *context);
 extern void undo_fake_foreign_function_call(os_context_t* context);
 extern void arrange_return_to_c_function(
     os_context_t *, call_into_lisp_lookalike, lispobj);
